@@ -1,6 +1,7 @@
 from .skeleton import (
   SkeletonTask, UnshardedSkeletonMergeTask, 
-  ShardedSkeletonMergeTask
+  ShardedSkeletonMergeTask, DeleteSkeletonFilesTask,
+  TransferSkeletonFilesTask, ShardedFromUnshardedSkeletonMergeTask
 )
 from .mesh import (
   MeshTask, MeshManifestPrefixTask, 
@@ -9,7 +10,7 @@ from .mesh import (
   MultiResUnshardedMeshMergeTask,
   MultiResShardedMeshMergeTask,
   MultiResShardedFromUnshardedMeshMergeTask,
-  TransferMeshFilesTask,
+  TransferMeshFilesTask, DeleteMeshFilesTask
 )
 from .image import (
   HyperSquareConsensusTask, #HyperSquareTask,
