@@ -56,7 +56,7 @@ class Tuple2(click.ParamType):
 
 @click.group()
 @click.option("-p", "--parallel", default=1, help="Run with this number of parallel processes. If 0, use number of cores.")
-@click.version_option(version="3.0.0")
+@click.version_option(version="3.1.0")
 @click.pass_context
 def main(ctx, parallel):
   """
@@ -501,6 +501,24 @@ def mesh_sharded_merge(
   tq = TaskQueue(normalize_path(queue))
   tq.insert(tasks, parallel=parallel)
 
+@meshgroup.command("rm")
+@click.argument("path")
+@click.option('--queue', required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue", type=str)
+@click.option('--magnitude', default=2, help="Split up the work with 10^(magnitude) prefix based tasks. Default: 2 (100 tasks)")
+@click.option('--dir', 'mesh_dir', default=None, help="Target this directory instead of the one indicated in the info file.")
+@click.pass_context
+def mesh_rm(ctx, path, queue, magnitude, mesh_dir):
+  """
+  Delete mesh files.
+  """
+  path = cloudfiles.paths.normalize(path)
+  tasks = tc.create_mesh_deletion_tasks(
+    path, magnitude=magnitude, mesh_dir=mesh_dir
+  )
+
+  parallel = int(ctx.obj.get("parallel", 1))
+  tq = TaskQueue(normalize_path(queue))
+  tq.insert(tasks, parallel=parallel)
 
 @meshgroup.group("spatial-index")
 def spatialindexgroup():
@@ -751,6 +769,62 @@ def delete_images(
   tq = TaskQueue(normalize_path(queue))
   tq.insert(tasks, parallel=parallel)
 
+@skeletongroup.command("rm")
+@click.argument("path")
+@click.option('--queue', required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue", type=str)
+@click.option('--magnitude', default=2, help="Split up the work with 10^(magnitude) prefix based tasks. Default: 2 (100 tasks)")
+@click.option('--dir', 'skel_dir', default=None, help="Target this directory instead of the one indicated in the info file.")
+@click.pass_context
+def skeleton_rm(ctx, path, queue, magnitude, skel_dir):
+  """
+  Delete skeleton files.
+  """
+  path = cloudfiles.paths.normalize(path)
+  tasks = tc.create_skeleton_deletion_tasks(
+    path, magnitude=magnitude, skel_dir=skel_dir
+  )
+
+  parallel = int(ctx.obj.get("parallel", 1))
+  tq = TaskQueue(normalize_path(queue))
+  tq.insert(tasks, parallel=parallel)
+
+
+@skeletongroup.command("xfer")
+@click.argument("src")
+@click.argument("dest")
+@click.option('--queue', required=True, help="AWS SQS queue or directory to be used for a task queue. e.g. sqs://my-queue or ./my-queue. See https://github.com/seung-lab/python-task-queue", type=str)
+@click.option("--sharded", is_flag=True, default=False, help="Generate shard fragments instead of outputing mesh fragments.", show_default=True)
+@click.option("--dir", "skel_dir", type=str, default=None, help="Write skeletons into this directory instead of the one indicated in the info file.")
+@click.option('--magnitude', default=2, help="Split up the work with 10^(magnitude) prefix based tasks.", show_default=True)
+@click.pass_context
+def skel_xfer(
+  ctx, src, dest, queue,
+  sharded, skel_dir, magnitude
+):
+  """
+  Transfer skeletons to another location.
+  Enables conversion of unsharded to sharded
+  as well.
+  """
+  src = cloudfiles.paths.normalize(src)
+  dest = cloudfiles.paths.normalize(dest)
+  cv_src = CloudVolume(src)
+
+  if not cv_src.skeleton.meta.is_sharded() and sharded:
+    tasks = tc.create_sharded_skeletons_from_unsharded_tasks(
+      src, dest, skel_dir=skel_dir,
+    )
+  else:
+    tasks = tc.create_xfer_skeleton_tasks(
+      src, dest,
+      skel_dir=skel_dir, 
+      magnitude=magnitude,
+    )
+
+  parallel = int(ctx.obj.get("parallel", 1))
+  tq = TaskQueue(normalize_path(queue))
+  tq.insert(tasks, parallel=parallel)
+
 @main.group("design")
 def designgroup():
   """
